RaidGUIControlKickMuteWidget.INFO_PADDING = 4
RaidGUIControlKickMuteWidget.WEAPON_ICON_H = 32
RaidGUIControlKickMuteWidget.WEAPON_PANEL_W = 104
RaidGUIControlKickMuteWidget.GRENADE_PANEL_W = 78
RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE = 36
RaidGUIControlKickMuteWidget.SKILL_LOCKED_Y_OFFSET = -3

-- Hooks

Hooks:PreHook(RaidGUIControlKickMuteWidget, "init",
	"PlayerInfo_RaidGUIControlKickMuteWidget_pre_init",
	function(self, parent, params)
		if not RaidGUIControlKickMuteWidget._HEIGHT_ORIGINAL then
			RaidGUIControlKickMuteWidget._HEIGHT_ORIGINAL = RaidGUIControlKickMuteWidget.HEIGHT
		end
		if not RaidGUIControlKickMuteWidget._HIGHLIGHT_LINE_H_ORIGINAL then
			RaidGUIControlKickMuteWidget._HIGHLIGHT_LINE_H_ORIGINAL = RaidGUIControlKickMuteWidget.HIGHLIGHT_LINE_H
		end

		self._player_info_add_background = PlayerInfo:GetConfigOption("add_background")
		self._player_info_show_nationality = PlayerInfo:GetConfigOption("show_nationality")
		self._player_info_show_level = PlayerInfo:GetConfigOption("show_level")
		self._player_info_show_weapons = PlayerInfo:GetConfigOption("show_weapons")
		self._player_info_include_grenade = PlayerInfo:GetConfigOption("include_grenade")
		self._player_info_show_warcry = PlayerInfo:GetConfigOption("show_warcry")
		self._player_info_show_boost = PlayerInfo:GetConfigOption("show_boost")
		self._player_info_show_skills = PlayerInfo:GetConfigOption("show_skills")

		self._player_info_second_row = (self._player_info_show_weapons or self._player_info_show_warcry or self._player_info_show_boost or self._player_info_show_skills)

		if self._player_info_second_row then
			RaidGUIControlKickMuteWidget.HEIGHT = RaidGUIControlKickMuteWidget._HEIGHT_ORIGINAL * 2
			RaidGUIControlKickMuteWidget.HIGHLIGHT_LINE_H = RaidGUIControlKickMuteWidget._HIGHLIGHT_LINE_H_ORIGINAL * 3
		else
			RaidGUIControlKickMuteWidget.HEIGHT = RaidGUIControlKickMuteWidget._HEIGHT_ORIGINAL
			RaidGUIControlKickMuteWidget.HIGHLIGHT_LINE_H = RaidGUIControlKickMuteWidget._HIGHLIGHT_LINE_H_ORIGINAL
		end
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "init",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post_init",
	function(self, parent, params)
		self:_create_peer_nationality_icon()
		self:_create_peer_level_text()
		self:_create_peer_weapons_panel()
		self:_create_peer_skills_panel()
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "_create_panel",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post__create_panel",
	function(self, parent, params)
		if self._player_info_add_background then
			self._background = self._object:bitmap({
				layer = -2,
				name = "background",
				x = 1,
				texture = tweak_data.gui.icons[HUDChat.BACKGROUND_IMAGE].texture,
				texture_rect = tweak_data.gui.icons[HUDChat.BACKGROUND_IMAGE].texture_rect
			})
		end
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "_fit_size",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post__fit_size",
	function(self)
		if self._player_info_add_background then
			self._background:set_w(self._object:w())
			self._background:set_h(self._object:h() - 2)
		end
		self:_reposition_peer_nationality_icon()
		self:_reposition_peer_level_text()
		self:_reposition_peer_name_text()
		self:_reposition_peer_weapons()
		self:_reposition_peer_skills()
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "set_peer",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post_set_peer",
	function(self, peer, mute_button, kick_button)
		self:_recreate_peer_nationality_icon()
		self:_recreate_peer_level_text()
		self:_recreate_peer_weapons()
		self:_recreate_peer_skills()
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "_create_kick_button",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post__create_kick_button",
	function(self)
		if self._player_info_second_row then
			self._kick_button:set_center_y(self._object:h() / 4)
		end
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "_create_mute_button",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post__create_mute_button",
	function(self)
		if self._player_info_second_row then
			self._mute_button:set_center_y(self._object:h() / 4)
		end
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "_create_raid_files_button",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post__create_raid_files_button",
	function(self)
		if self._player_info_second_row then
			self._raid_files_button:set_center_y(self._object:h() / 4)
		end
	end
)

Hooks:PostHook(RaidGUIControlKickMuteWidget, "_refresh_vote_kick_button",
	"PlayerInfo_RaidGUIControlKickMuteWidget_post__refresh_vote_kick_button",
	function(self)
		if self._is_local_player then
			self._kick_button:set_visible(false)
		end
	end
)

local calculate_width_original = RaidGUIControlKickMuteWidget.calculate_width
function RaidGUIControlKickMuteWidget:calculate_width(...)
	local w1 = 0
	if self._player_info_show_nationality and self._nationality_icon then
		w1 = w1 + self._nationality_icon:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
	end
	if self._player_info_show_level and self._level_text then
		w1 = w1 + self._level_text:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
	end

	local w2 = 0
	if self._player_info_show_weapons and self._weapons_panel then
		w2 = w2 + self._weapons_panel:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
	end
	if self._skills_panel and (self._player_info_show_warcry or self._player_info_show_boost or self._player_info_show_skills) then
		w2 = w2 + self._skills_panel:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
	end

	-- yeah, have fun reading this, i dont even know how to explain, but ill try..
	-- w1 is simple, cause calculate_width_original includes NAME_X and its padding already.
	-- while w (row2) includes here, what calculate_width_original would kinda do. we dont want another padding after the last object.
	-- the 0 thing kinda also includes that row2 isnt there, when both options are off
	-- i hope that makes sense
	return math.max(w1 + calculate_width_original(self, ...),
		w2 and (RaidGUIControlKickMuteWidget.NAME_X + w2 - RaidGUIControlKickMuteWidget.INFO_PADDING) or 0)
end

-- custom functions

function RaidGUIControlKickMuteWidget:set_is_local_player()
	self._is_local_player = true
	local session = managers.network:session()
	if session then
		self:set_peer(session:local_peer())
	end
	self._mute_button:set_visible(false)
	self._kick_button:set_visible(false)
	if self._raid_files_button and not session then -- workaround for 'open in raid files'
		self._raid_files_button:set_visible(false) -- hide, clicking it crashes in offline mode, because peer is nil
	end
	self:_recreate_peer_skills()
	self:_recreate_peer_weapons()
end

function RaidGUIControlKickMuteWidget:_reposition_peer_name_text()
	local x = RaidGUIControlKickMuteWidget.NAME_X

	if self._level_text then
		local _, _, w = self._level_text:text_rect()
		x = self._level_text:x() + w + RaidGUIControlKickMuteWidget.INFO_PADDING
	elseif self._nationality_icon then
		x = self._nationality_icon:x() + self._nationality_icon:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
	end

	self._name:set_x(x)
end

-- Nationality Icon

function RaidGUIControlKickMuteWidget:_create_peer_nationality_icon()
	if not self._player_info_show_nationality then
		return
	end

	self:_recreate_peer_nationality_icon()
end

function RaidGUIControlKickMuteWidget:_reposition_peer_nationality_icon()
	if self._nationality_icon then
		self._nationality_icon:set_x(RaidGUIControlKickMuteWidget.NAME_X)
		self._nationality_icon:set_center_y(self._name:center_y())
	end

	self:_reposition_peer_level_text()
end

function RaidGUIControlKickMuteWidget:_recreate_peer_nationality_icon()
	if self._nationality_icon then
		self._object:remove(self._nationality_icon)
		self._nationality_icon = nil
	end

	if not self._is_local_player and not self._peer then
		return
	end

	if not self._player_info_show_nationality then
		-- if option not enabled at all, skip here
		return
	end

	local playerData = nil

	if self._is_local_player then
		playerData = PlayerInfo:GetLocalPlayerInfo()
	else
		playerData = PlayerInfo:GetPlayerDataByPeer(self._peer)
	end
	if not (playerData and playerData.nationality) then
		return
	end

	local tweak = tweak_data.gui.icons['nationality_small_' .. playerData.nationality]
	if tweak and tweak.texture and tweak.texture_rect then
		self._nationality_icon = self._object:bitmap({
			name = "player_info_nationality",
			w = 32,
			h = 32,
			texture = tweak.texture,
			texture_rect = tweak.texture_rect,
		})
	end

	self:_reposition_peer_nationality_icon()
end

-- Level Text

function RaidGUIControlKickMuteWidget:_create_peer_level_text()
	if not self._player_info_show_level then
		return
	end

	self:_recreate_peer_level_text()
end

function RaidGUIControlKickMuteWidget:_reposition_peer_level_text()
	if self._level_text then
		local x = RaidGUIControlKickMuteWidget.NAME_X

		if self._player_info_show_nationality and self._nationality_icon then
			x = x + self._nationality_icon:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
		end

		self._level_text:set_x(x)
		self._level_text:set_center_y(self._name:center_y())
		local _, _, w = self._level_text:text_rect()
		self._level_text:set_w(w)
	end

	self:_reposition_peer_name_text()
end

function RaidGUIControlKickMuteWidget:_recreate_peer_level_text()
	if self._level_text then
		self._object:remove(self._level_text)
		self._level_text = nil
	end

	if not self._is_local_player and not self._peer then
		return
	end

	if not self._player_info_show_level then
		-- if option not enabled at all, skip here
		return
	end

	local playerData = nil

	if self._is_local_player then
		playerData = PlayerInfo:GetLocalPlayerInfo()
	else
		playerData = PlayerInfo:GetPlayerDataByPeer(self._peer)
	end
	if not (playerData and playerData.level) then
		return
	end

	self._level_text = self._object:label({
		name = "player_info_level",
		text = tostring(playerData.level),
		vertical = "center",
		h = self._name:h(),
		font = RaidGUIControlKickMuteWidget.NAME_FONT,
		font_size = RaidGUIControlKickMuteWidget.NAME_FONT_SIZE,
		color = RaidGUIControlKickMuteWidget.NAME_FONT_COLOR_INACTIVE,
	})

	self:_reposition_peer_level_text()
end

-- Weapons

function RaidGUIControlKickMuteWidget:_create_peer_weapons_panel()
	if not self._player_info_show_weapons then
		return
	end

	local w = (RaidGUIControlKickMuteWidget.WEAPON_PANEL_W * 2)

	if self._player_info_include_grenade then
		w = w + RaidGUIControlKickMuteWidget.GRENADE_PANEL_W + RaidGUIControlKickMuteWidget.INFO_PADDING
	end

	self._weapons_panel = self._object:panel({
		name = "player_info_weapons",
		w = w,
		h = self._name:h()
	})

	self:_reposition_peer_weapons()
	self:_recreate_peer_weapons()
end

function RaidGUIControlKickMuteWidget:_recreate_peer_weapons()
	if not self._weapons_panel then
		return
	end

	self._weapons_panel:clear()

	if not self._player_info_show_weapons then
		-- if weapons not enabled at all, skip here
		return
	end

	if not self._is_local_player and not self._peer then
		return
	end

	local playerData = nil

	if self._is_local_player then
		playerData = PlayerInfo:GetLocalPlayerInfo()
	else
		playerData = PlayerInfo:GetPlayerDataByPeer(self._peer)
	end
	if not (playerData and playerData.weapons.primary and playerData.weapons.secondary and playerData.weapons.grenade) then
		return
	end

	local on_mouse_exit_callback = callback(self, self, "_hide_skill_tooltip")

	-- PRIMARY

	local x = 0
	local primary = PlayerInfo.weapons_index[playerData.weapons.primary.factory_id]
	local last_icon = self._weapons_panel:image({
		name = "player_info_weapon_primary",
		x = x,
		w = primary.texture_rect[3] * (RaidGUIControlKickMuteWidget.WEAPON_ICON_H / primary.texture_rect[4]),
		h = RaidGUIControlKickMuteWidget.WEAPON_ICON_H,
		texture = primary.texture,
		texture_rect = primary.texture_rect,
		on_mouse_enter_callback = callback(self, self, "_show_skill_tooltip",
			{
				name_id = primary.name_id,
			}),
		on_mouse_exit_callback = on_mouse_exit_callback,
	})
	last_icon:set_center_y(self._weapons_panel:h() / 2)

	-- SECONDARY

	x = last_icon:right() + RaidGUIControlKickMuteWidget.INFO_PADDING
	local secondary = PlayerInfo.weapons_index[playerData.weapons.secondary.factory_id]
	last_icon = self._weapons_panel:image({
		name = "player_info_weapon_secondary",
		x = x,
		w = secondary.texture_rect[3] * (RaidGUIControlKickMuteWidget.WEAPON_ICON_H / secondary.texture_rect[4]),
		h = RaidGUIControlKickMuteWidget.WEAPON_ICON_H,
		texture = secondary.texture,
		texture_rect = secondary.texture_rect,
		on_mouse_enter_callback = callback(self, self, "_show_skill_tooltip",
			{
				name_id = secondary.name_id,
			}),
		on_mouse_exit_callback = on_mouse_exit_callback,
	})
	last_icon:set_center_y(self._weapons_panel:h() / 2)

	-- GRENADE

	if self._player_info_include_grenade then
		x = last_icon:right() + RaidGUIControlKickMuteWidget.INFO_PADDING
		local grenade = PlayerInfo.grenades_index[playerData.weapons.grenade]
		last_icon = self._weapons_panel:panel({
			name = "player_info_weapon_grenade_wrap",
			x = x,
			w = RaidGUIControlKickMuteWidget.GRENADE_PANEL_W,
			h = RaidGUIControlKickMuteWidget.WEAPON_ICON_H,
			on_mouse_enter_callback = callback(self, self, "_show_skill_tooltip",
				{
					name_id = grenade.name_id,
				}),
			on_mouse_exit_callback = on_mouse_exit_callback,
		})
		last_icon:set_center_y(self._weapons_panel:h() / 2)
		local grenade_icon = last_icon:image({
			name = "player_info_weapon_grenade",
			halign = "center",
			valign = "center",
			w = grenade.texture_rect[3] * (RaidGUIControlKickMuteWidget.WEAPON_ICON_H / grenade.texture_rect[4]),
			h = RaidGUIControlKickMuteWidget.WEAPON_ICON_H,
			texture = grenade.texture,
			texture_rect = grenade.texture_rect,
			on_mouse_enter_callback = callback(self, self, "_show_skill_tooltip",
				{
					name_id = grenade.name_id,
				}),
			on_mouse_exit_callback = on_mouse_exit_callback,
		})
		grenade_icon:set_center_x(last_icon:w() / 2)
		grenade_icon:set_center_y(last_icon:h() / 2)
	end
end

function RaidGUIControlKickMuteWidget:_reposition_peer_weapons()
	if self._weapons_panel then
		self._weapons_panel:set_x(RaidGUIControlKickMuteWidget.NAME_X)
		self._weapons_panel:set_y(RaidGUIControlKickMuteWidget.HEIGHT / 2)
	end
end

-- Skills

function RaidGUIControlKickMuteWidget:_create_peer_skills_panel()
	if not (self._player_info_show_warcry or self._player_info_show_boost or self._player_info_show_skills) then
		return
	end

	local w = 0
	if self._player_info_show_warcry then
		w = w + RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE + RaidGUIControlKickMuteWidget.INFO_PADDING
	end
	if self._player_info_show_boost then
		w = w + RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE + RaidGUIControlKickMuteWidget.INFO_PADDING
	end
	if self._player_info_show_skills then
		w = w + ((RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE + RaidGUIControlKickMuteWidget.INFO_PADDING) * 8)
	end
	w = w - RaidGUIControlKickMuteWidget.INFO_PADDING

	self._skills_panel = self._object:panel({
		name = "player_info_skills",
		w = w,
		h = self._name:h()
	})

	self:_reposition_peer_skills()
	self:_recreate_peer_skills()
end

function RaidGUIControlKickMuteWidget:_recreate_peer_skills()
	if not self._skills_panel then
		return
	end

	self._skills_panel:clear()

	if not (self._player_info_show_warcry or self._player_info_show_boost or self._player_info_show_skills) then
		-- if no option enabled at all, skip here
		return
	end

	if not self._is_local_player and not self._peer then
		-- not ready yet
		return
	end

	local playerData = nil

	if self._is_local_player then
		playerData = PlayerInfo:GetLocalPlayerInfo()
	else
		playerData = PlayerInfo:GetPlayerDataByPeer(self._peer)
	end
	if not (playerData and playerData.skills and playerData.warcry and playerData.level) then
		-- not ready yet
		return
	end

	local on_mouse_exit_callback = callback(self, self, "_hide_skill_tooltip")
	local callback_locked = callback(self, self, "_show_skill_tooltip",
		{ name_id = "playerinfo_slot_locked" })
	local callback_not_set = callback(self, self, "_show_skill_tooltip",
		{ name_id = "playerinfo_slot_not_set" })

	local x = 0
	local last_icon = nil

	-- WARCRY

	if self._player_info_show_warcry then
		local warcry_icon_name = tweak_data.warcry[playerData.warcry].hud_icon
		local warcry_tweak = tweak_data.skilltree.skills['warcry_' .. playerData.warcry]
		last_icon = self._skills_panel:image({
			name = "warcry_icon",
			x = x,
			w = RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE,
			h = RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE,
			color = tweak_data.skilltree.skill_category_colors[SkillTreeTweakData.TYPE_WARCRY],
			texture = tweak_data.gui.icons[warcry_icon_name].texture,
			texture_rect = tweak_data.gui.icons[warcry_icon_name].texture_rect,
			on_mouse_enter_callback = callback(self, self, "_show_skill_tooltip",
				{
					name_id = warcry_tweak and warcry_tweak.name_id or playerData.warcry,
				}),
			on_mouse_exit_callback = on_mouse_exit_callback,
		})
		last_icon:set_center_y(self._skills_panel:h() / 2)
	end

	-- BOOST

	if self._player_info_show_boost then
		local boost = nil
		local boost_level = nil
		for _, data in ipairs(playerData.skills) do
			local tweak = tweak_data.skilltree.skills[data.name]
			if tweak and tweak.upgrades_type == SkillTreeTweakData.TYPE_BOOSTS then
				boost = data.name
				boost_level = data.level
			end
		end

		-- dirty workaround for boosts not having upgrades for level 2 and 3
		-- this istn accurate either, but this way it simply looks a little nicer, i think
		if not self._is_local_player and boost_level and boost_level == 1 then
			local tweak = tweak_data.skilltree.skills[boost]
			if table.size(tweak.upgrades[2]) == 0 and table.size(tweak.upgrades[3]) == 0 then -- excludes mida's touch from this hack, which has upgrades for 2 and 3.
				boost_level = 2
			end
		end

		x = last_icon and last_icon:right() or 0
		if boost and boost_level then
			last_icon = self:_recreate_peer_skill_icon(
				self._skills_panel, x,
				RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE, RaidGUIControlKickMuteWidget.INFO_PADDING,
				tweak_data.gui.colors.raid_skill_boost,
				boost, boost_level,
				on_mouse_exit_callback
			)
		else
			local locked = playerData.level < 5
			local icon = tweak_data.gui.icons[locked and "ico_locker" or "skill_slot_unlocked"]
			last_icon = self._skills_panel:image({
				name = "empty_boost",
				x = x + RaidGUIControlKickMuteWidget.INFO_PADDING,
				w = RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE,
				h = RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE,
				color = locked and tweak_data.gui.colors.raid_dark_grey or tweak_data.gui.colors.raid_dirty_white,
				texture = icon.texture,
				texture_rect = icon.texture_rect,
				on_mouse_enter_callback = locked and callback_locked or callback_not_set,
				on_mouse_exit_callback = on_mouse_exit_callback,
			})
			last_icon:set_center_y((self._skills_panel:h() / 2) +
				(locked and RaidGUIControlKickMuteWidget.SKILL_LOCKED_Y_OFFSET or 0))
		end
	end

	-- SKILLS

	if self._player_info_show_skills then
		local has_skills = 0
		for _, data in ipairs(playerData.skills) do
			local tweak = tweak_data.skilltree.skills[data.name]
			if tweak and tweak.upgrades_type ~= SkillTreeTweakData.TYPE_BOOSTS then
				has_skills = has_skills + 1
				x = last_icon and last_icon:right() or 0
				last_icon = self:_recreate_peer_skill_icon(
					self._skills_panel, x,
					RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE, RaidGUIControlKickMuteWidget.INFO_PADDING,
					tweak_data.gui.colors.raid_skill_talent,
					data.name, data.level,
					on_mouse_exit_callback
				)
			end
		end

		-- fill up with empty skill slots

		local max_skills = table.size(tweak_data.skilltree.skill_talent_tiers)
		local empty_skills = max_skills - has_skills
		local j = 0
		while j < empty_skills do
			j = j + 1
			local locked = tweak_data.skilltree.skill_talent_tiers[has_skills + j] > playerData.level
			local icon = tweak_data.gui.icons[locked and "ico_locker" or "skill_slot_unlocked"]
			x = last_icon and last_icon:right() or 0
			last_icon = self._skills_panel:image({
				name = "empty_skill_" .. tostring(j),
				x = x + RaidGUIControlKickMuteWidget.INFO_PADDING,
				w = RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE,
				h = RaidGUIControlKickMuteWidget.SKILL_ICON_SIZE,
				color = locked and tweak_data.gui.colors.raid_dark_grey or tweak_data.gui.colors.raid_dirty_white,
				texture = icon.texture,
				texture_rect = icon.texture_rect,
				on_mouse_enter_callback = locked and callback_locked or callback_not_set,
				on_mouse_exit_callback = on_mouse_exit_callback,
			})
			last_icon:set_center_y((self._skills_panel:h() / 2) +
				(locked and RaidGUIControlKickMuteWidget.SKILL_LOCKED_Y_OFFSET or 0))
		end
	end
end

function RaidGUIControlKickMuteWidget:_reposition_peer_skills()
	if not self._skills_panel then
		return
	end

	local x = RaidGUIControlKickMuteWidget.NAME_X

	if self._player_info_show_weapons and self._weapons_panel then
		x = x + self._weapons_panel:w() + RaidGUIControlKickMuteWidget.INFO_PADDING
	end

	self._skills_panel:set_x(x)
	self._skills_panel:set_y(RaidGUIControlKickMuteWidget.HEIGHT / 2)
end

function RaidGUIControlKickMuteWidget:_recreate_peer_skill_icon(panel, x, icon_size, padding, color, id, level,
																on_mouse_exit_callback)
	local icon = tweak_data.skilltree:get_skill_icon_tiered(id)
	local gui_skill = tweak_data.gui:get_full_gui_data(icon)
	local tweak = tweak_data.skilltree.skills[id]

	local on_mouse_enter_callback = callback(self, self, "_show_skill_tooltip", {
		name_id = tweak and tweak.name_id or id,
		level = level,
		max_level = 4
	})

	local result = panel:image({
		name = "skill_icon_" .. tostring(id) .. "_bg",
		x = x + padding,
		w = icon_size,
		h = icon_size,
		color = tweak_data.gui.colors.raid_white,
		texture = gui_skill.texture,
		texture_rect = gui_skill.texture_rect,
		on_mouse_enter_callback = on_mouse_enter_callback,
		on_mouse_exit_callback = on_mouse_exit_callback,
	})
	result:set_center_y(panel:h() / 2)

	local fill_wrap = panel:panel({
		x = x + padding,
		w = icon_size,
		h = icon_size / 4 * level,
	})
	fill_wrap:set_bottom(result:bottom())

	local fill = fill_wrap:image({
		name = "skill_icon_" .. tostring(id) .. "_fill",
		w = icon_size,
		h = icon_size,
		color = math.lerp(tweak_data.gui.colors.raid_skill_unapplied, color, 1 / 4 * level),
		texture = gui_skill.texture,
		texture_rect = gui_skill.texture_rect,
		on_mouse_enter_callback = on_mouse_enter_callback,
		on_mouse_exit_callback = on_mouse_exit_callback,
	})
	fill:set_bottom(fill_wrap:h())

	return result
end

-- Tooltips

-- params: name_id, (level), (max_level)
function RaidGUIControlKickMuteWidget:_show_skill_tooltip(params)
	if not managers.menu_component then
		return
	end

	local raid_menu_main_menu_gui = managers.menu_component._raid_menu_main_menu_gui
	if not raid_menu_main_menu_gui then
		return
	end

	local action_title = raid_menu_main_menu_gui._widget_action_title

	if action_title then
		local name = self:translate(params.name_id, true)
		action_title:set_text(name)
		if params.level and params.max_level then
			action_title:set_text(string.format('%s [%s/%s]', name, params.level, params.max_level))
		end
	end
end

function RaidGUIControlKickMuteWidget:_hide_skill_tooltip()
	local raid_menu_main_menu_gui = managers.menu_component._raid_menu_main_menu_gui
	if not raid_menu_main_menu_gui then
		return
	end

	if raid_menu_main_menu_gui._widget_action_title then
		raid_menu_main_menu_gui._widget_action_title:set_text("")
	end
end
