PlayerInfo = PlayerInfo or {}
if not PlayerInfo.setup then
	PlayerInfo._path = ModPath
	PlayerInfo._lua_path = ModPath .. "Lua/"
	PlayerInfo._config_defaults = ModPath .. "default_values.json"
	PlayerInfo._config_path = SavePath .. "PlayerInfo.json"
	PlayerInfo._config = {}

	PlayerInfo._player_data_template = {
		name = 'Unknown',
		class = nil,
		nationality = nil,
		warcry = nil,
		peer_id = nil,
		level = 0,
		skills = {},
		weapons = {
			primary = nil,
			secondary = nil,
			grenade = nil,
			melee = nil, -- TODO ?
		},
		clothing = { -- TODO ?
			head = nil,
			upper = nil,
			lower = nil,
		},
		character = nil,      -- TODO ?
		concealment_modifier = nil, -- TODO ?

		-- internal
		_upgrades = {},
		_outfit_string = nil,
		--_outfit_version = -1,
	}

	PlayerInfo._player_data = {
		[1] = deep_clone(PlayerInfo._player_data_template),
		[2] = deep_clone(PlayerInfo._player_data_template),
		[3] = deep_clone(PlayerInfo._player_data_template),
		[4] = deep_clone(PlayerInfo._player_data_template) -- local player
	}

	PlayerInfo.upgrade_to_skills_with_level_index = {}
	PlayerInfo.weapons_index = {}
	PlayerInfo.grenades_index = {}
	PlayerInfo.skills_sorted = {}

	function PlayerInfo:Index()
		-- build upgrade -> skill/level index, for faster player skill updates later
		for skill, data in pairs(tweak_data.skilltree.skills) do
			-- build upgrade_to_skills index
			for level, upgrades in ipairs(data.upgrades) do
				if type(upgrades) == "table" then -- ignores weapon skills
					for _, upgrade in ipairs(upgrades) do
						if not self.upgrade_to_skills_with_level_index[upgrade] then
							self.upgrade_to_skills_with_level_index[upgrade] = {}
						end
						self.upgrade_to_skills_with_level_index[upgrade][skill] = level
					end
				end
			end

			-- build weapons cache
			for weapon_name, weapon_data in pairs(tweak_data.weapon) do
				local definitions = tweak_data.upgrades.definitions[weapon_name]
				if definitions and definitions.factory_id and weapon_data.hud then
					self.weapons_index[definitions.factory_id] = {
						name_id = weapon_data.name_id,
						texture = tweak_data.gui.icons[weapon_data.hud.icon].texture,
						texture_rect = tweak_data.gui.icons[weapon_data.hud.icon].texture_rect,
					}
				end
			end

			-- build grenades cache
			for grenade_name, grenade_data in pairs(tweak_data.projectiles) do
				if grenade_data.throwable and tweak_data.weapon[grenade_name] and tweak_data.weapon[grenade_name].hud then
					self.grenades_index[grenade_name] = {
						name_id = grenade_data.name_id,
						texture = tweak_data.gui.icons[tweak_data.weapon[grenade_name].hud.icon].texture,
						texture_rect = tweak_data.gui.icons[tweak_data.weapon[grenade_name].hud.icon].texture_rect,
					}
				end
			end

			-- build sorted skills list (by required level)
			table.insert_sorted(self.skills_sorted,
				{
					name = skill,
					level_required = data.level_required
				},
				function(a, b)
					return (a.level_required or 0) < (b.level_required or 0)
				end
			)
		end
	end

	function PlayerInfo:SaveConfig()
		local file = io.open(self._config_path, "w+")
		if file then
			file:write(json.encode(self._config))
			file:close()
		end
	end

	function PlayerInfo:LoadConfig()
		self:LoadConfigDefaults()
		local file = io.open(self._config_path, "r")
		if file then
			local config = json.decode(file:read("*all"))
			file:close()
			if config and type(config) == "table" then
				for k, v in pairs(config) do
					self._config[k] = v
				end
			end
		end
	end

	function PlayerInfo:GetConfigOption(id)
		return self._config[id]
	end

	function PlayerInfo:SetConfigOption(id, value)
		if self._config[id] ~= value then
			self._config[id] = value
			return true
		end

		return false
	end

	function PlayerInfo:LoadConfigDefaults()
		local default_file = io.open(self._config_defaults)
		if default_file then
			self._config = json.decode(default_file:read("*all"))
			default_file:close()
		end
	end

	function PlayerInfo:ResetPlayer(teammate_id)
		self._player_data[teammate_id] = deep_clone(PlayerInfo._player_data_template)
	end

	function PlayerInfo:SetPlayerName(teammate_id, playerName)
		self._player_data[teammate_id].name = playerName
	end

	function PlayerInfo:SetPlayerLevel(teammate_id, level, queue_update)
		self._player_data[teammate_id].level = level

		if queue_update then
			self:QueuePlayerSkillsUpdate(teammate_id)
		end

		-- push update to ui, if shown
		self:PushToUI(teammate_id, "level")
	end

	function PlayerInfo:SetPlayerWarcry(teammate_id, warcry, queue_update)
		self._player_data[teammate_id].warcry = warcry

		if queue_update then
			self:QueuePlayerSkillsUpdate(teammate_id)
		end
	end

	function PlayerInfo:SetPlayerClass(teammate_id, className, queue_update)
		self._player_data[teammate_id].class = className

		if queue_update then
			self:QueuePlayerSkillsUpdate(teammate_id)
		end
	end

	function PlayerInfo:SetPeerId(teammate_id, peer_id, queue_update)
		self._player_data[teammate_id].peer_id = peer_id

		if queue_update then
			self:QueuePlayerSkillsUpdate(teammate_id)
		end
	end

	function PlayerInfo:ClearPlayerUpgrades(teammate_id)
		self._player_data[teammate_id]._upgrades = {}
		self._player_data[teammate_id].skills = {}
	end

	function PlayerInfo:UpsertPlayerUpgrade(teammate_id, category, upgrade, upgrade_level, queue_update)
		-- make sure category exists
		if not self._player_data[teammate_id]._upgrades[category] then
			self._player_data[teammate_id]._upgrades[category] = {}
		end

		-- insert upgrades entry
		table.insert(self._player_data[teammate_id]._upgrades[category], {
			upgrade = upgrade,
			level = upgrade_level
		})

		if queue_update then
			self:QueuePlayerSkillsUpdate(teammate_id)
		end
	end

	-- buffers the update call for 1 sec, so it wont be executed more often than neccessary
	function PlayerInfo:QueuePlayerSkillsUpdate(teammate_id)
		DelayedCalls:Add("PlayerInfo_update_skills_" .. teammate_id, 1, function()
			self:UpdatePlayerSkills(teammate_id)
		end)
	end

	-- buffers the update call for 1 sec, so it wont be executed more often than neccessary
	function PlayerInfo:QueuePlayerWeaponsUpdate(teammate_id)
		DelayedCalls:Add("PlayerInfo_update_weapons_" .. teammate_id, 1, function()
			self:UpdatePlayerWeapons(teammate_id)
		end)
	end

	function PlayerInfo:UpdatePlayerSkills(teammate_id)
		if self._player_data[teammate_id] and (self._player_data[teammate_id]._upgrades or teammate_id == 4) and self._player_data[teammate_id].class then
			-- regenerate skills list
			local new_skills = {}
			if teammate_id ~= 4 then
				-- peers: reconstruct skills by upgrades (with imprecise boost levels [except mida's])
				for category, values in pairs(self._player_data[teammate_id]._upgrades) do
					for _, upgrade in ipairs(values) do
						local id = category .. "_" .. upgrade.upgrade
						if tweak_data.upgrades:upgrade_has_levels(id) then
							id = id .. "_" .. upgrade.level
						end
						local skills = self.upgrade_to_skills_with_level_index[id]
						if skills then
							for skill, skill_level in pairs(skills) do
								local class_lock = tweak_data.skilltree.skills[skill].class_lock
								if not class_lock or table.contains(class_lock, self._player_data[teammate_id].class) then
									local current = new_skills[skill]
									if not current or current < skill_level then
										new_skills[skill] = skill_level
									end
								end
							end
						end
					end
				end
			else
				-- local player: get skills (with correct) levels directly
				for type_id, category in ipairs(managers.skilltree:get_character_skilltree() or {}) do
					if type_id == 2 or type_id == 3 then
						for skill_id, skill_data in pairs(category) do
							if skill_data.active then
								new_skills[skill_id] = skill_data.exp_tier
							end
						end
					end
				end
			end

			-- sort
			self._player_data[teammate_id].skills = {}
			for _, data in ipairs(self.skills_sorted) do
				if new_skills[data.name] then
					table.insert(self._player_data[teammate_id].skills,
						{
							name = data.name,
							level = new_skills[data.name]
						}
					)
				end
			end

			-- push update to ui, if shown
			self:PushToUI(teammate_id, "skills")
		end
	end

	function PlayerInfo:GetPlayerInfoAll()
		return self._player_data
	end

	function PlayerInfo:GetLocalPlayerInfo()
		return self._player_data[HUDManager and HUDManager.PLAYER_PANEL or 4]
	end

	function PlayerInfo:GetPlayerDataByPeer(peer)
		for _, data in ipairs(self._player_data) do
			if data.peer_id and data.peer_id == peer:id() then
				return data
			end
		end
		return nil
	end

	function PlayerInfo:UpdateLocalPlayerInfo()
		if HUDManager and managers.skilltree and managers.player and managers.warcry and managers.experience and managers.network then
			local teammate_id = HUDManager.PLAYER_PANEL or 4
			self:SetPlayerClass(teammate_id, managers.skilltree:get_character_profile_class()) -- bugs out skills, when switching characters, if not set here.
			self:SetPlayerWarcry(teammate_id, managers.warcry:get_active_warcry_name())
			self:SetPlayerLevel(teammate_id, managers.experience:current_level())
			self:UpdatePlayerSkills(teammate_id)
			self:UpdateLocalPlayerOutfit()
		end
	end

	function PlayerInfo:UpdateLocalPlayerOutfit()
		if not managers.blackmarket then
			return
		end

		local id = HUDManager and HUDManager.PLAYER_PANEL or 4
		self:SetPlayerOutfitString(id, managers.blackmarket:outfit_string())
		self:UpdatePlayerWeapons(id)
	end

	function PlayerInfo:SetPlayerOutfitString(teammate_id, outfit_string, outfit_version, outfit_signature, queue_update)
		self._player_data[teammate_id]._outfit_string = outfit_string
		if queue_update then
			self:QueuePlayerWeaponsUpdate(teammate_id)
		end
	end

	function PlayerInfo:UpdatePlayerWeapons(teammate_id)
		if not managers.blackmarket then
			return
		end

		local outfit_string = self._player_data[teammate_id]._outfit_string

		if not outfit_string then
			return
		end

		local outfit = managers.blackmarket:unpack_outfit_from_string(outfit_string)

		for key, path in pairs({
			['character'] = { 'character' },
			['skills'] = { 'class' },
			['character_customization_nationality'] = { 'nationality' },
			['primary'] = { 'weapons', 'primary' },
			['secondary'] = { 'weapons', 'secondary' },
			['grenade'] = { 'weapons', 'grenade' },
			-- TODO ?
			['melee_weapon'] = { 'weapons', 'melee' },
			['character_customization_head'] = { 'clothing', 'head' },
			['character_customization_upper'] = { 'clothing', 'upper' },
			['character_customization_lower'] = { 'clothing', 'lower' },
			['concealment_modifier'] = { 'concealment_modifier' },
			--['armor'] = { 'armor' },
			--['armor_current'] = { 'armor_current' },
			--['armor_current_state'] = { 'armor_current_state' },
			--['deployable'] = { 'deployable' },
			--['deployable_count'] = { 'deployable_count' },
		}) do
			local current = self._player_data[teammate_id]
			for i, next in ipairs(path) do
				if i < #path then
					current = current[next] or {}
				else
					current[next] = outfit[key]
				end
			end
		end

		-- push update to ui, if shown
		self:PushToUI(teammate_id, "nationality")
		self:PushToUI(teammate_id, "weapons")
	end

	function PlayerInfo:PushToUI(teammate_id, func)
		if managers.menu_component and managers.menu_component._raid_menu_main_menu_gui and managers.menu_component._raid_menu_main_menu_gui._widgets and table.size(managers.menu_component._raid_menu_main_menu_gui._widgets) > 0 then
			for _, widget in ipairs(managers.menu_component._raid_menu_main_menu_gui._widgets) do
				if ((widget._is_local_player and teammate_id == 4) or (widget._peer and self._player_data[teammate_id].peer_id and (widget._peer:id() == self._player_data[teammate_id].peer_id))) then
					if func == 'nationality' then
						widget:_recreate_peer_nationality_icon()
					end
					if func == 'level' then
						widget:_recreate_peer_level_text()
					end
					if func == 'skills' then
						widget:_recreate_peer_skills()
					end
					if func == 'weapons' then
						widget:_recreate_peer_weapons()
					end
				end
			end
		end
	end

	PlayerInfo:LoadConfig()
	PlayerInfo.setup = true
end
